using UnityEngine;
using System.Collections;

namespace Mod
{
    public class Water : Liquid
    {
        public const string ID = "Water";
        public Water() { Color = new UnityEngine.Color(0.25f, 0.45f, 0.78f, 0.5f); }

        public override void OnEnterLimb(LimbBehaviour limb) {}
        public override void OnUpdate(BloodContainer container) {}
        public override void OnEnterContainer(BloodContainer container) {}
        public override void OnExitContainer(BloodContainer container) {}
    }

    public class CoughSyrup : Liquid
    {
        public const string ID = "Cough Syrup";

        public CoughSyrup() { Color = new UnityEngine.Color(0.53f, 0f, 0.92f); }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PersonBehaviour person = limb.Person;
            LimbBehaviour[] limbs0 = person.Limbs;
            if (limb = limbs0[0])
            {
                LimbBehaviour[] limbs1 = new LimbBehaviour[3];
                limbs1[0] = limbs0[0];
                limbs1[1] = limbs0[1];
                limbs1[2] = limbs0[2];

                foreach (LimbBehaviour l in limbs1)
                {
                    l.ImpactPainMultiplier = 0;
                }
            }
        }

        public override void OnUpdate(BloodContainer container) {}
        public override void OnEnterContainer(BloodContainer container) {}
        public override void OnExitContainer(BloodContainer container) {}
    }

    public class Susium : Liquid
    {
        public const string ID = "Susium";

        public Susium() { Color = new UnityEngine.Color(0.75f, 1f, 0.56f, 0.7f); }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PersonBehaviour person = limb.Person;
            LimbBehaviour[] limbs = person.Limbs;
            foreach (LimbBehaviour l in limbs)
            {
                l.ExplodeLiquidAmount *= 50f;
                l.Crush();
            }
        }

        public override void OnUpdate(BloodContainer container) {}
        public override void OnEnterContainer(BloodContainer container) {}
        public override void OnExitContainer(BloodContainer container) {}
    }

    public class Lean : Liquid
    {
        public const string ID = "Lean";

        public Lean() { Color = new UnityEngine.Color(0.47f, 0f, 0.72f); }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PersonBehaviour person = limb.Person;
            LimbBehaviour[] limbs = person.Limbs;
            foreach (LimbBehaviour l in limbs)
            {
                l.ImpactPainMultiplier *= 3f;
                l.FakeUprightForce /= 10f;
                l.DoStumble = true;
            }
        }

        public override void OnUpdate(BloodContainer container) {}
        public override void OnEnterContainer(BloodContainer container) {}
        public override void OnExitContainer(BloodContainer container) {}
    }

    public class Soda : Liquid
    {
        public const string ID = "Soda";

        public Soda() { Color = new UnityEngine.Color(0.83f, 1f, 0.98f, 0.25f); }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PersonBehaviour person = limb.Person;
            LimbBehaviour[] limbs = person.Limbs;
            foreach (LimbBehaviour l in limbs)
            {
                l.ImpactPainMultiplier *= 1.5f;
                l.FakeUprightForce /= 2f;
            }
        }

        public override void OnUpdate(BloodContainer container) {}
        public override void OnEnterContainer(BloodContainer container) {}
        public override void OnExitContainer(BloodContainer container) {}
    }

    public class SuperLean : Liquid
    {
        public const string ID = "Super Lean";

        public SuperLean() { Color = new UnityEngine.Color(0.13f, 0f, 0.32f); }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            PersonBehaviour person = limb.Person;
            LimbBehaviour[] limbs = person.Limbs;
            foreach (LimbBehaviour l in limbs)
            {
                l.ImpactPainMultiplier *= 5f;
                l.FakeUprightForce /= 30f;
                l.DoStumble = true;
            }
        }

        public override void OnUpdate(BloodContainer container) {}
        public override void OnEnterContainer(BloodContainer container) {}
        public override void OnExitContainer(BloodContainer container) {}
    }

    public class Gasoline : Liquid
    {
        public const string ID = "Gasoline";

        public Gasoline() { Color = new UnityEngine.Color(0f, 0.38f, 0.41f, 0.8f); }


        public override void OnUpdate(BloodContainer container) 
        {
            if (container.gameObject.GetComponent<PhysicalBehaviour>().Temperature >= 70) Combust(container);
            else if (container.gameObject.GetComponent<PhysicalBehaviour>().Charge > 0) Combust(container);
        }

        public override void OnEnterLimb(LimbBehaviour limb) {}

        public override void OnEnterContainer(BloodContainer container) {}

        public override void OnExitContainer(BloodContainer container) {}

        void Combust(BloodContainer container)
        {
            ExplosionCreator.Explode(container.transform.position, 1.5f);
            container.RemoveLiquid(Liquid.GetLiquid("Gasoline"), 0.5f);
        }
    }
}