using UnityEngine;
using System.Collections;
using UnityEngine.UI;
using TMPro;

namespace Mod
{
    public class ElectricityGenerator : MonoBehaviour
    {
        public GameObject Wheel;
        Rigidbody2D wheelRB;

        void Start()
        {
            wheelRB = Wheel.GetComponent<Rigidbody2D>();
        }

        void Update()
        {
            float speed = wheelRB.angularVelocity/20;
            if (speed < 0) speed = -speed;
            GetComponent<PhysicalBehaviour>().Charge = speed;
        }
    }

    public class RotationalSpeedometer : MonoBehaviour
    {
        public GameObject Wheel;
        public int minSpeed, maxSpeed;
        public TextMeshPro textMesh;
        Rigidbody2D wheelRB;

        void Start()
        {
            wheelRB = Wheel.GetComponent<Rigidbody2D>();
        }

        void Update()
        {
            textMesh.enabled = true;
            float speed = wheelRB.angularVelocity/50;
            speed = Mathf.RoundToInt(speed);
            if (speed > maxSpeed) speed = maxSpeed;
            if (speed < minSpeed) speed = minSpeed;
            textMesh.text = speed+" RPM";
            if (textMesh.text == "") textMesh.text = "0 RPM"; 
        }
    }

    public class TorqueGenerator : MonoBehaviour
    {
        public GameObject Wheel;
        Rigidbody2D wheelRB;

        void Start()
        {
            wheelRB = Wheel.GetComponent<Rigidbody2D>();
        }

        void Update()
        {
            float heat = GetComponent<PhysicalBehaviour>().Temperature;
            heat = -heat;
            if (heat > 0) heat = 0;
            if (heat > PhysicalBehaviour.AmbientTemperature) wheelRB.angularVelocity = heat;
        }
    }

    public class Combustor : MonoBehaviour
    {
        public FlaskBehaviour container;
        bool combusting = false;
        SpriteRenderer rend;
        Sprite on = ModAPI.LoadSprite("sprites/2/1.png"), off = ModAPI.LoadSprite("sprites/2/0.png");


        void Start()
        {
            container.Capacity *= 2;
            rend = GetComponent<SpriteRenderer>();
            on = ModAPI.LoadSprite("sprites/2/1.png");
            off = ModAPI.LoadSprite("sprites/2/0.png");
        }

        void Update()
        {
            if (combusting)
            {
                rend.sprite = on;
                if (container.GetAmount(Liquid.GetLiquid("NITRO")) >= 0.5f)
                {
                    ExplosionCreator.Explode(new Vector3(transform.position.x + 0.21f, transform.position.y + 0.45f, transform.position.z), 0.8f);
                    container.RemoveLiquid(Liquid.GetLiquid("NITRO"), 0.5f);
                    GetComponent<PhysicalBehaviour>().Temperature += 8;
                }
                else if (container.GetAmount(Liquid.GetLiquid("Gasoline")) >= 0.5f)
                {
                    ExplosionCreator.Explode(new Vector3(transform.position.x + 0.21f, transform.position.y + 0.45f, transform.position.z), 1.5f);
                    container.RemoveLiquid(Liquid.GetLiquid("Gasoline"), 0.5f);
                    GetComponent<PhysicalBehaviour>().Temperature += 10;
                }
            }
            else if (!combusting) rend.sprite = off;
        }

        public void Use(ActivationPropagation activation)
        {
            combusting = !combusting;
        }
    }

    public class Electrometer : MonoBehaviour
    {
        public int maxCharge;
        public TextMeshPro textMesh;
        PhysicalBehaviour phys;

        void Start()
        {
            phys = GetComponent<PhysicalBehaviour>();
        }

        void Update()
        {
            textMesh.enabled = true;
            float charge = phys.Charge;
            charge = Mathf.RoundToInt(charge);
            if (charge > maxCharge) charge = maxCharge;
            textMesh.text = charge+"";
            if (textMesh.text == "") textMesh.text = "0"; 
        }
    }
}