using UnityEngine;
using System.Collections;
using UnityEngine.UI;

namespace Mod
{
    public class Mod
    {
        public static void Main()
        {
            //liquids
            ModAPI.RegisterLiquid(Susium.ID, new Susium());
            ModAPI.RegisterLiquid(CoughSyrup.ID, new CoughSyrup());
            ModAPI.RegisterLiquid(Lean.ID, new Lean());
            ModAPI.RegisterLiquid(Soda.ID, new Soda());
            ModAPI.RegisterLiquid(SuperLean.ID, new SuperLean());
            ModAPI.RegisterLiquid(Gasoline.ID, new Gasoline());
            ModAPI.RegisterLiquid(Water.ID, new Water());

            //mixing
            var lifeDeathMixture = new LiquidMixInstructions(
                Liquid.GetLiquid("LIFE SERUM"),
                Liquid.GetLiquid("INSTANT DEATH POISON"),                     
                Liquid.GetLiquid(Susium.ID)
            );
            lifeDeathMixture.RatePerSecond = 1f;

            LiquidMixingController.MixInstructions.Add(lifeDeathMixture);

            var susiumMendingMixture = new LiquidMixInstructions(
                Liquid.GetLiquid("MENDING SERUM"),
                Liquid.GetLiquid(Susium.ID),
                Liquid.GetLiquid(CoughSyrup.ID)
            );
            susiumMendingMixture.RatePerSecond = 1f;

            LiquidMixingController.MixInstructions.Add(susiumMendingMixture);

            var coughSyrupSodaMixture = new LiquidMixInstructions(
                Liquid.GetLiquid(CoughSyrup.ID),
                Liquid.GetLiquid(Soda.ID),
                Liquid.GetLiquid(Lean.ID)
            );
            coughSyrupSodaMixture.RatePerSecond = 1f;

            LiquidMixingController.MixInstructions.Add(coughSyrupSodaMixture);

            var sodaLeanMixture = new LiquidMixInstructions(
                Liquid.GetLiquid(Soda.ID),
                Liquid.GetLiquid(Lean.ID),
                Liquid.GetLiquid(SuperLean.ID)
            );
            sodaLeanMixture.RatePerSecond = 1f;

            LiquidMixingController.MixInstructions.Add(sodaLeanMixture);

            var nitroOilMixture = new LiquidMixInstructions(
                Liquid.GetLiquid("OIL"),
                Liquid.GetLiquid("NITRO"),
                Liquid.GetLiquid(Gasoline.ID)
            );
            nitroOilMixture.RatePerSecond = 1f;

            LiquidMixingController.MixInstructions.Add(nitroOilMixture);



            //items
            
            //Soda Bottle [Misc]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Bottle"),
                NameOverride = "Soda Bottle -PPGA",
                DescriptionOverride = "a bottle filled with soda",
                CategoryOverride = ModAPI.FindCategory("Misc."),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/0/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/0/0.png");
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        FlaskBehaviour flask = Instance.GetComponent<FlaskBehaviour>();
                        flask.StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = Soda.ID,
                            Amount = 2.8f
                        };
                    }
                    GameObject.Destroy(Instance.GetComponent<RandomSpriteBehaviour>());
                }
            });

            //Electricity Generator [Machinery]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Electricity Generator -PPGA",
                DescriptionOverride = "a generator that turns rotational force into electricity",
                CategoryOverride = ModAPI.FindCategory("Machinery"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/1/t.png"),
                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/1/0.png");

                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.AddComponent<BoxCollider2D>();

                    ElectricityGenerator behaviour = Instance.GetOrAddComponent<ElectricityGenerator>();
                    GameObject wheel = ModAPI.CreatePhysicalObject("wheelthing", ModAPI.LoadSprite("sprites/1/1.png")).gameObject;
                    foreach(Collider2D col in wheel.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    wheel.AddComponent<CircleCollider2D>();
                    wheel.transform.SetParent(Instance.transform);
                    wheel.transform.localScale = new Vector2(1, 1);
                    float n = 0.281f;
                    Vector3 scale = Instance.transform.localScale;
                    if (scale.x < 0) n = -0.281f;
                    wheel.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.3f;
                    wheel.transform.position = new Vector3(Instance.transform.position.x + n, Instance.transform.position.y, Instance.transform.position.z);
                    HingeJoint2D joint = wheel.GetOrAddComponent<HingeJoint2D>();
                    joint.connectedBody = Instance.GetComponent<Rigidbody2D>();
                    behaviour.Wheel = wheel;
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1f;
                    }
                }
            });

            //Combustor [Machinery]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Combustor -PPGA",
                DescriptionOverride = "a machine that combusts nitroglycerine/petrol when activated.",
                CategoryOverride = ModAPI.FindCategory("Machinery"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/2/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/2/0.png");

                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.FixColliders();
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Combustor behaviour = Instance.GetOrAddComponent<Combustor>();
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1.2f;
                        behaviour.container = Instance.GetOrAddComponent<FlaskBehaviour>();
                    }
                }
            });

            //Sledge Hammer [Melee]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Sledge Hammer -PPGA",
                DescriptionOverride = "a hammer with a flat head, used for smashing human heads.",
                CategoryOverride = ModAPI.FindCategory("Melee"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/3/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/3/0c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/3/1c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/3/0.png");
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1.9f;
                    }
                }
            });

            //Nail Hammer [Melee]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Nail Hammer -PPGA",
                DescriptionOverride = "a hammer used for removing nails, also for smashing human heads.",
                CategoryOverride = ModAPI.FindCategory("Melee"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/4/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/4/0c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/4/1c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/4/0.png");
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        var phys = Instance.GetComponent<PhysicalBehaviour>();
                        phys.TrueInitialMass = 1.1f;
                    }
                }
            });

            //Lean Bottle [Misc]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Bottle"),
                NameOverride = "Lean Bottle -PPGA",
                DescriptionOverride = "a bottle filled with lean",
                CategoryOverride = ModAPI.FindCategory("Misc."),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/5/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/5/0.png");
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        FlaskBehaviour flask = Instance.GetComponent<FlaskBehaviour>();
                        flask.StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = Lean.ID,
                            Amount = 2.8f
                        };
                    }
                    GameObject.Destroy(Instance.GetComponent<RandomSpriteBehaviour>());
                }
            });

            //Torque Generator [Machinery]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Torque Generator -PPGA",
                DescriptionOverride = "a generator that turns heat into rotational force",
                CategoryOverride = ModAPI.FindCategory("Machinery"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/6/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/6/0.png");

                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.AddComponent<BoxCollider2D>();

                    TorqueGenerator behaviour = Instance.GetOrAddComponent<TorqueGenerator>();
                    GameObject wheel = ModAPI.CreatePhysicalObject("wheelthing", ModAPI.LoadSprite("sprites/6/1.png")).gameObject;
                    foreach(Collider2D col in wheel.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    wheel.AddComponent<CircleCollider2D>();
                    wheel.transform.SetParent(Instance.transform);
                    wheel.transform.localScale = new Vector2(1, 1);
                    float n = ((ModAPI.PixelSize * 9) - (ModAPI.PixelSize / 2));
                    Vector3 scale = Instance.transform.localScale;
                    if (scale.x < 0) n = -n;
                    wheel.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.3f;
                    wheel.transform.position = new Vector3(Instance.transform.position.x + n, Instance.transform.position.y - ((ModAPI.PixelSize * 4) + (ModAPI.PixelSize / 2)), Instance.transform.position.z);
                    HingeJoint2D joint = wheel.GetOrAddComponent<HingeJoint2D>();
                    joint.connectedBody = Instance.GetComponent<Rigidbody2D>();
                    behaviour.Wheel = wheel;

                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1.7f;
                    }
                }
            });

            //Rotational Speedometer [Machinery]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Thermometer"),
                NameOverride = "Rotational Speedometer -PPGA",
                DescriptionOverride = "shows how fast a wheel is going.",
                CategoryOverride = ModAPI.FindCategory("Machinery"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/7/t.png"),
                AfterSpawn = (Instance) =>
                {
                    ModAPI.KeepExtraObjects();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/7/0.png");

                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.AddComponent<BoxCollider2D>();

                    //afterspawn code
                    RotationalSpeedometer behaviour = Instance.GetOrAddComponent<RotationalSpeedometer>();
                    behaviour.textMesh = Instance.GetComponent<ThermometerBehaviour>().TextMesh;
                    GameObject.Destroy(Instance.GetComponent<ThermometerBehaviour>());
                    behaviour.minSpeed = -9999;
                    behaviour.maxSpeed = 9999;

                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.8f;
                        GameObject wheel = ModAPI.CreatePhysicalObject("wheelthing", ModAPI.LoadSprite("sprites/7/1.png")).gameObject;
                        foreach(Collider2D col in wheel.GetComponents<Collider2D>()) GameObject.Destroy(col);
                        wheel.AddComponent<CircleCollider2D>();
                        wheel.transform.SetParent(Instance.transform);
                        wheel.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
                        float n = ModAPI.PixelSize * 9;
                        Vector3 scale = Instance.transform.localScale;
                        if (scale.x < 0) n = -n;
                        wheel.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.1f;
                        wheel.transform.position = new Vector3(Instance.transform.position.x + n, Instance.transform.position.y, Instance.transform.position.z);
                        HingeJoint2D joint = wheel.GetOrAddComponent<HingeJoint2D>();
                        joint.connectedBody = Instance.GetComponent<Rigidbody2D>();
                        behaviour.Wheel = wheel;
                    }
                }
            });

            //Hydraulic Ball [Misc]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Bowling Ball"),
                NameOverride = "Hydraulic Ball -PPGA",
                DescriptionOverride = "a ball that changes its size based on how much liquid is inside.",
                CategoryOverride = ModAPI.FindCategory("Misc."),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/8/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/8/0.png");

                    //afterspawn code
                    Instance.GetOrAddComponent<HydraulicBallBehaviour>();
                    Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.5f;
                    Instance.GetComponent<PhysicalBehaviour>().Resizable = false;
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                    GameObject.Destroy(Instance.GetComponent<RandomSpriteBehaviour>());
                }
            });

            //Traffic Cone [Misc]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Traffic Cone -PPGA",
                DescriptionOverride = "its hollow!",
                CategoryOverride = ModAPI.FindCategory("Misc."),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/9/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/9/0c.png", 10);                        Instance.AddComponent<PolygonCollider2D>();
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/9/1c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/9/2c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/9/0.png");
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Plastic");
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.6f;
                        Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
                    }
                }
            });

            //Electrometer [Machinery]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Thermometer"),
                NameOverride = "Electrometer -PPGA",
                DescriptionOverride = "shows how much charge it has.",
                CategoryOverride = ModAPI.FindCategory("Machinery"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/10/t.png"),
                AfterSpawn = (Instance) =>
                {
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        ModAPI.KeepExtraObjects();

                        //sprite
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/10/0.png");

                        //collider
                        foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                        Instance.AddComponent<BoxCollider2D>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.8f;
                        Electrometer behaviour = Instance.GetOrAddComponent<Electrometer>();
                        behaviour.textMesh = Instance.GetComponent<ThermometerBehaviour>().TextMesh;
                        GameObject.Destroy(Instance.GetComponent<ThermometerBehaviour>());
                        behaviour.maxCharge = 999999;
                    }
                }
            });

            //Empty Canister [Chemistry]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Empty Canister -PPGA",
                DescriptionOverride = "a canister, you can put any liquid inside it then pour it out!",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/11/et.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/0.png");

                    //afterspawn code
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                    var behaviour = Instance.GetOrAddComponent<Canister>();
                    behaviour.liquidDrop = ModAPI.LoadSprite("sprites/11/p.png");
                    FlaskBehaviour flask = Instance.GetOrAddComponent<FlaskBehaviour>();
                    flask.Capacity = 3;
                }
            });

            //Gasoline Canister [Chemistry]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Gasoline Canister -PPGA",
                DescriptionOverride = "a canister full of gasoline, you can pour it out!",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/11/gt.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/0.png");

                    //afterspawn code
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                    var behaviour = Instance.GetOrAddComponent<Canister>();
                    behaviour.liquidDrop = ModAPI.LoadSprite("sprites/11/p.png");
                    FlaskBehaviour flask = Instance.GetOrAddComponent<FlaskBehaviour>();
                    flask.Capacity = 3;
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.7f;
                        flask.StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = Gasoline.ID,
                            Amount = 3f
                        };
                    }
                }
            });

            //Nitroglycerine Canister [Chemistry]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Nitroglycerine Canister -PPGA",
                DescriptionOverride = "a canister full of nitroglycerine, you can pour it out!",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/11/nt.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/0.png");

                    //afterspawn code
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                    var behaviour = Instance.GetOrAddComponent<Canister>();
                    behaviour.liquidDrop = ModAPI.LoadSprite("sprites/11/p.png");
                    FlaskBehaviour flask = Instance.GetOrAddComponent<FlaskBehaviour>();
                    flask.Capacity = 3;
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.7f;
                        flask.StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = Nitroglycerine.ID,
                            Amount = 3f
                        };
                    }
                }
            });

            //Water Canister [Chemistry]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Water Canister -PPGA",
                DescriptionOverride = "a canister full of water, you can pour it out!",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/11/wt.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/0.png");

                    //afterspawn code
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                    var behaviour = Instance.GetOrAddComponent<Canister>();
                    behaviour.liquidDrop = ModAPI.LoadSprite("sprites/11/p.png");
                    FlaskBehaviour flask = Instance.GetOrAddComponent<FlaskBehaviour>();
                    flask.Capacity = 3;
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.7f;
                        flask.StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = Water.ID,
                            Amount = 3f
                        };
                    }
                }
            });

            //Oil Canister [Chemistry]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Oil Canister -PPGA",
                DescriptionOverride = "a canister full of oil, you can pour it out!",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/11/ot.png"),
                AfterSpawn = (Instance) =>
                {
                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/c.png", 10);
                    Instance.AddComponent<PolygonCollider2D>();

                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/11/0.png");

                    //afterspawn code
                    Instance.GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                    var behaviour = Instance.GetOrAddComponent<Canister>();
                    behaviour.liquidDrop = ModAPI.LoadSprite("sprites/11/p.png");
                    FlaskBehaviour flask = Instance.GetOrAddComponent<FlaskBehaviour>();
                    flask.Capacity = 3;
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        Instance.GetComponent<PhysicalBehaviour>().TrueInitialMass = 0.7f;
                        flask.StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = Oil.ID,
                            Amount = 3f
                        };
                    }
                }
            });


            //Match Stick [Misc]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Stick"),
                NameOverride = "Match Stick -PPGA",
                DescriptionOverride = "a match stick.",
                CategoryOverride = ModAPI.FindCategory("Misc."),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/12/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/12/0.png");

                    //collider
                    foreach(Collider2D col in Instance.GetComponents<Collider2D>()) GameObject.Destroy(col);
                    Instance.AddComponent<BoxCollider2D>();

                    var behaviour = Instance.GetOrAddComponent<MatchStick>();
                    var phys = Instance.GetComponent<PhysicalBehaviour>();
                    phys.Properties = ModAPI.FindPhysicalProperties("Wood");
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        phys.TrueInitialMass = 0.1f;
                    }
                }
            });

            //Water Bottle [Misc]
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Bottle"),
                NameOverride = "Water Bottle -PPGA",
                DescriptionOverride = "a bottle filled with water",
                CategoryOverride = ModAPI.FindCategory("Misc."),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/13/t.png"),
                AfterSpawn = (Instance) =>
                {
                    //sprite
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("sprites/13/0.png");
                    if (Instance.GetComponent<AfterSpawn>() == null)
                    {
                        Instance.AddComponent<AfterSpawn>();

                        //afterspawn code
                        FlaskBehaviour flask = Instance.GetComponent<FlaskBehaviour>();
                        flask.StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = Water.ID,
                            Amount = 2.8f
                        };
                    }
                    GameObject.Destroy(Instance.GetComponent<RandomSpriteBehaviour>());
                }
            });

            //other

            ModAPI.Register<SpaceController>();
        }
    }

    public class SpaceController : MonoBehaviour
    {
        public bool InSpace = false;

        void Start()
        {
            gameObject.name = "SpaceController";
            var set = MapConfig.Instance.Settings.ShallowClone();
            set.Gravity = 0;
            if (MapLoaderBehaviour.CurrentMap.UniqueIdentity == "265b1c13-c372-4ea4-8448-09731f0b0985")
            {
                MapConfig.Instance.ApplySettings(set);
                InSpace = true;
            }
            else InSpace = false;
        }
    }

    public class AfterSpawn : MonoBehaviour {}
}