using UnityEngine;
using System.Collections;

public class MatchStick : MonoBehaviour
{
    bool Lit, Burned = false;

    void Update()
    {
        if (Lit && !GetComponent<PhysicalBehaviour>().OnFire) Lit = false;
        else if (!Lit && GetComponent<PhysicalBehaviour>().OnFire) Lit = true;
        if (Lit && GetComponent<PhysicalBehaviour>().BurnProgress < 1) GetComponent<PhysicalBehaviour>().BurnProgress += 0.001f;
        if (GetComponent<PhysicalBehaviour>().BurnProgress >= 1) Burned = true;
    }

    void OnCollisionEnter2D(Collision2D collision) 
    { 
        if (GetComponent<Rigidbody2D>().velocity.magnitude >= 6) Light(); 
        if (GetComponent<PhysicalBehaviour>().OnFire) collision.otherCollider.gameObject.GetComponent<PhysicalBehaviour>().Ignite(true);
    }

    void OnCollisionStay2D(Collision2D collision) 
    { 
        if (GetComponent<Rigidbody2D>().velocity.magnitude >= 6) Light();
        if (GetComponent<PhysicalBehaviour>().OnFire) collision.otherCollider.gameObject.GetComponent<PhysicalBehaviour>().Ignite(true);
    }

    void Light()
    {
        if (!Lit && !Burned)
        {
            if (Random.Range(0.01f, 0.99f) > 0.5f)
            {
                var vector = new Vector2(transform.position.x+(ModAPI.PixelSize*2f), transform.position.y);
                ModAPI.CreateParticleEffect("FuseBlown", vector);
                GetComponent<PhysicalBehaviour>().Ignite();
                GetComponent<PhysicalBehaviour>().BurnIntensity = 10f;
            }
        }
    }
}

public class Canister : MonoBehaviour
{
    public Sprite liquidDrop;
    //GameObject thingObj;
    //LiquidContainerController thing;
    float liquidShowY;
    GameObject liquidShow;
    Vector2 lsScale;
    bool Open = false, Dropping = false, CanDrop = false, Empty = false;
    Transform barrel;
    bool[] type;
    FlaskBehaviour container;
    Color liquidColor;

    void Start()
    {
        Vector2 bPosition = new Vector2(ModAPI.PixelSize*8, ModAPI.PixelSize*6);
        barrel = new GameObject("Barrel").transform;
        barrel.SetParent(transform);
        barrel.localPosition = bPosition;
        container = gameObject.GetComponent<FlaskBehaviour>();
        type = new bool[5];

        //thing = gameObject.AddComponent<LiquidContainerController>();
        //thing.LowFidelityLiquid = ModAPI.FindSpawnable("Flask").Prefab.gameObject.GetComponent<LiquidContainerController>().LowFidelityLiquid;
        //thing.Container = new Bounds(new Vector2(-(ModAPI.PixelSize*3.5f), -(ModAPI.PixelSize*1.5f)), new Vector2(-ModAPI.PixelSize, (ModAPI.PixelSize*3.5f)));

        liquidShow = new GameObject("Liquid Visibility Thingy");
        liquidShow.AddComponent<SpriteRenderer>();
        liquidShow.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
        liquidShow.GetComponent<SpriteRenderer>().sprite = liquidDrop;
        liquidShow.transform.SetParent(transform);
        liquidShow.transform.localRotation = Quaternion.identity;
        liquidShow.transform.localPosition = new Vector2(-(ModAPI.PixelSize*3.5f), -(ModAPI.PixelSize*1.5f));
        liquidShowY = -(ModAPI.PixelSize*3.5f);
    }

    void Update()
    {
        //behaviour
        if (Open && CanDrop && !Dropping) StartCoroutine(Drop());

        //CanDrop
        if (Vector2.Angle(Vector2.down, transform.right) < 60 && transform.localScale.x > 0) CanDrop = true;
        else if (Vector2.Angle(Vector2.down, -transform.right) < 60 && transform.localScale.x < 0) CanDrop = true;
        else CanDrop = false;
        
        //Get Properties
        liquidColor = container.GetComputedColor();
        if (container.GetAmount(Liquid.GetLiquid("Gasoline")) >= 0.1f) type[0] = true;
        if (container.GetAmount(Liquid.GetLiquid("NITRO")) >= 0.1f) type[1] = true;
        if (container.GetAmount(Liquid.GetLiquid("TRITIUM")) >= 0.1f) type[2] = true;
        if (container.GetAmount(Liquid.GetLiquid("EXOTIC LIQUID")) >= 0.1f) type[3] = true;
        if (container.GetAmount(Liquid.GetLiquid("Water")) >= 0.1f) type[4] = true;

        //Empty
        if (container.ScaledLiquidAmount == 0) Empty = true;
        else Empty = false;

        //thingy
        //thing.FillPercentage = container.TotalLiquidAmount;
        //thing.Color = liquidColor;

        if (container.TotalLiquidAmount >= 0 && container.TotalLiquidAmount < 0.14f)
        {
            lsScale = new Vector2(1, 0);
            liquidShowY = -(ModAPI.PixelSize*5f);
        }
        else if (container.TotalLiquidAmount >= 0.14f && container.TotalLiquidAmount < 0.28f)
        {
            lsScale = new Vector2(1, 0.5f);
            liquidShowY = -(ModAPI.PixelSize*4.5f);
        }
        else if (container.TotalLiquidAmount >= 0.28f && container.TotalLiquidAmount < 0.42f)
        {
            lsScale = new Vector2(1, 1f);
            liquidShowY = -(ModAPI.PixelSize*4f);
        }
        else if (container.TotalLiquidAmount >= 0.42f && container.TotalLiquidAmount < 0.57f)
        {
            lsScale = new Vector2(1, 1.5f);
            liquidShowY = -(ModAPI.PixelSize*3.5f);
        }
        else if (container.TotalLiquidAmount >= 0.57f && container.TotalLiquidAmount < 0.71f)
        {
            lsScale = new Vector2(1, 2f);
            liquidShowY = -(ModAPI.PixelSize*3f);
        }
        else if (container.TotalLiquidAmount >= 0.71f && container.TotalLiquidAmount < 0.85f)
        {
            lsScale = new Vector2(1, 2.5f);
            liquidShowY = -(ModAPI.PixelSize*2.5f);
        }
        else if (container.TotalLiquidAmount >= 0.85f && container.TotalLiquidAmount < 1f)
        {
            lsScale = new Vector2(1, 3f);
            liquidShowY = -(ModAPI.PixelSize*2f);
        }
        else if (container.TotalLiquidAmount >= 1)
        {
            lsScale = new Vector2(1, 3.5f);
            liquidShowY = -(ModAPI.PixelSize*1.5f);
        }

        liquidShow.transform.localScale = lsScale;
        liquidShow.transform.localPosition = new Vector2(-(ModAPI.PixelSize*3.5f), liquidShowY);
        liquidShow.GetComponent<SpriteRenderer>().color = liquidColor;
    }

    public void Use()
    {
        Open = !Open;
    }

    IEnumerator Drop()
    {
        yield return new WaitForSeconds(0);
        if (Open && CanDrop && !Empty)
        {
            //setting variables
            Dropping = true;
            container.Drain(0.01f);
            
            //creating liquid particle
            GameObject drop = ModAPI.CreatePhysicalObject("Liquid Particle", liquidDrop);
            
            //setting the color
            drop.GetComponent<SpriteRenderer>().color = liquidColor;
            
            //setting the position
            drop.transform.SetParent(transform);
            drop.transform.localPosition = barrel.localPosition;
            drop.transform.parent = null;
            
            //setting the collider
            foreach(Collider2D col in drop.GetComponents<Collider2D>()) GameObject.Destroy(col);
            drop.AddComponent<CircleCollider2D>();
            drop.GetComponent<CircleCollider2D>().radius = 0.02f;
            
            //ignoring collisions between the particle and the canister
            Physics2D.IgnoreCollision(GetComponent<Collider2D>(), drop.GetComponent<Collider2D>(), true);
            
            //adding debris component
            drop.AddComponent<DebrisComponent>();
            
            //giving it the velocity of the canister
            drop.GetComponent<Rigidbody2D>().velocity = GetComponent<Rigidbody2D>().velocity;

            //new material
            PhysicsMaterial2D mat = new PhysicsMaterial2D();
            if (type[0] || type[1] || type[4]) mat.friction = 0.8f;
            if (type[2]) mat.friction = 0.5f;
            if (type[3]) mat.friction = 0.6f;
            
            //properties
            var phys = drop.GetComponent<PhysicalBehaviour>();
            
            if (type[4]) phys.Properties.Conducting = true;
            else phys.Properties.Conducting = false;
            if (type[3] || type[2]) phys.Properties.MagneticAttractionIntensity = 0.2f;
            else phys.Properties.MagneticAttractionIntensity = 0;
            if (type[0] || type[1])
            {
                phys.Properties.Flammability = 1;
                phys.Properties.BurningTemperatureThreshold = 76;
                phys.Properties.Burnrate = 1;
            }
            else
            {
                phys.Properties.Flammability = 0;
                phys.Properties.BurningTemperatureThreshold = 9999999;
                phys.Properties.Burnrate = 0;
            }
            if (type[0]) phys.Properties.Buoyancy = 3;
            else if (type[1]) phys.Properties.Buoyancy = 0.5f;
            else if (type[2] || type[3] || type[4]) phys.Properties.Buoyancy = 4;
            phys.Properties.Brittleness = 0.9f;
            phys.Properties.BulletSpeedAbsorptionPower = 0.05f;
            phys.Properties.Softness = 0;

            //no sounds.
            phys.Properties.StabSound = null;
            phys.Properties.SlidingLoop = null;
            phys.Properties.HardImpact = null;
            phys.Properties.SoftImpact = null;
            phys.Properties.SizzleSounds = null;
                
            //setting the physics material
            phys.Properties.PhysicMaterial = mat;
                
            //setting some properties
            phys.TrueInitialMass = 0.01f;
            phys.DisplayBloodDecals = false;

            //giving it the liquid particle component
            LiquidParticle particle = drop.AddComponent<LiquidParticle>();

            //other
            if (type[2]) ModAPI.CreateLight(drop.transform, Color.green, 0.2f, 0.4f);

            //giving it the properties of the liquid particle
            particle.type = type;
            yield return new WaitForSeconds(0.005f);
            StartCoroutine(Drop());
        }
        else Dropping = false;
    }
}

public class LiquidParticle : MonoBehaviour
{
    public bool[] type;
    bool lit = false;

    void Update()
    {
        //properties
        if (type[3]) GetComponent<PhysicalBehaviour>().Charge = 10; //Exotic Liquid
        if (GetComponent<PhysicalBehaviour>().OnFire && !lit) if (type[0] || type[1]) Light();
        else if (!GetComponent<PhysicalBehaviour>().OnFire && lit) if (type[0] || type[1]) lit = false;
    }

    void Light()
    {
        lit = true;
        if (type[0])
        {
            GetComponent<PhysicalBehaviour>().Ignite(true);
            GetComponent<PhysicalBehaviour>().BurnIntensity = 1.3f;
            //BurnNearbyLiquidParticles(0.05f);
            Destroy(gameObject, 0.5f);
        }
        else if (type[1])
        {
            ExplosionCreator.Explode(transform.position, 0.05f);
            //BurnNearbyLiquidParticles(0.1f);
            Destroy(gameObject);
        }
    }
    
    void BurnNearbyLiquidParticles(float radius = 0.05f)
    {
        var col = Physics2D.OverlapCircle(transform.position, radius);
        var obj = col.gameObject;
        if (obj.GetComponent<LiquidParticle>() != null) obj.GetComponent<LiquidParticle>().Light();
    }
}